
#include <p18f4550.h>

#include "eeprom.h"

int EE_write(unsigned char address, unsigned char data)
  {
    // Datasheet page 91
    EEDATA = data;     // EEDATA holds the 8-bit data for read/write
    EEADR  = address;  // EEADR register holds the address of the EEPROM location being accessed.
    
    EECON1bits.EEPGD = 0; // When clear, operations will access the data EEPROM memory.
    EECON1bits.CFGS = 0;  // When CFGS is clear, the EEPGD bit selects either program Flash or data EEPROM memory.
    EECON1bits.WREN = 1;  // The WREN bit, when set, will allow a write operation.
    INTCONbits.GIE = 0;   // disable interrupts
    EECON2 = 0x55;
    EECON2 = 0x0AA;
    
    EECON1bits.WR = 1;   // The WR control bit initiates write operations.
  
    // Since global interrupts are off,
    // let's stay inside this function
    // and wait for completion.
    while (EECON1bits.WR);
    
    if(EECON1bits.WRERR)
      {
        EECON1bits.WREN = 0;
        INTCONbits.GIE = 1;  // enable interrupts
        return 0;
      }
        
    EECON1bits.WREN = 0;
    INTCONbits.GIE = 1;  // enable interrupts
    return 1;
}

unsigned char EE_read(unsigned char address)
  {
    // the book, "Advanced PIC microcontroller projects in C"
    // implies that one should wait 20ms between sussive reads.
    // However, page 93 of datasheet does not indicate this.
    
    int value = 0;
    EEADR = address;
    EECON1bits.CFGS = 0;
    EECON1bits.EEPGD = 0;
    EECON1bits.RD = 1;
    value = EEDATA;
    
    return(value);
  }
