
//#include "user.h"
#include "toggle.h"

#include <p18f4550.h>
#include "user.h"

extern volatile unsigned int tmr0_overflow_counter;


#define SLEEP_TIME 4  

#define BT_HOME     0
#define BT_PRS      1
#define BT_REL      2
#define BT_PRS_WAIT 3
#define BT_REL_WAIT 4

void toggle_init(TOGGLE_BUTTON *ptr, unsigned char value)
  {
    ptr->timer_value = 0;
    ptr->state       = BT_HOME;
    ptr->value       = value;
    ptr->status      = TOGGLE_UP;
  }

unsigned char toggle_get_status(TOGGLE_BUTTON *ptr)
  {
    return ptr->status;
  }

unsigned char toggle_get_value(TOGGLE_BUTTON *ptr)
  {
    return ptr->value;
  }
  
void toggle_set_value(TOGGLE_BUTTON *ptr, unsigned char value)
  {
    ptr->value = value;
  }

    
void toggle_update(TOGGLE_BUTTON *ptr, unsigned char PORT)
  {
    switch (ptr->state)
      {
        case BT_HOME:
            if (PORT == 0) 
              {                   
                //button is pressed
                ptr->state = BT_PRS_WAIT;
                ptr->timer_value = tmr0_overflow_counter;
              }
          break;
          
        case BT_PRS_WAIT:
            if (PORT == 0)
              {
                //button is still pressed
                //we will wait until the timer expires
                if ((tmr0_overflow_counter - ptr->timer_value)>SLEEP_TIME) 
                    ptr->state = BT_PRS;
              }
            else 
                //button is not pressed, this is a false press
                ptr->state = BT_HOME;
          break;
                
        case BT_PRS:
            // now the button has been pressed for a period of time.  
            // we conclude it really is pressed and not a false press.
            ptr->status = TOGGLE_DOWN;
            if (PORT == 1)
              {
                // now the button is up again.
                ptr->state = BT_REL_WAIT;
                ptr->timer_value = tmr0_overflow_counter;
              }
          break;
          
        case BT_REL_WAIT:
            if (PORT == 1)
              {
                //button is still up
                //we will wait until the timer expires
                if ((tmr0_overflow_counter - ptr->timer_value)>SLEEP_TIME) 
                    ptr->state = BT_REL;

              }
            else
                //button is no longer up, this was a false release condition
                ptr->state = BT_PRS;
          break;        

        case BT_REL:
                //button is really up.
                ptr->status = TOGGLE_UP;
                ptr->state = BT_HOME;
                ptr->value = !(ptr->value);
              
          break;
      }       
  }