

#include <string.h>

#ifndef MY_LINUX_TESTING
  #include <p18f4550.h>
  #include "user.h"
#endif

#include "my_sd_card.h"


static unsigned long sector = 0;

static unsigned int offset = 0; /* bytes */


#pragma udata dataBuffer
static BYTE buffer[512];
#pragma udata


unsigned long SD_get_sector_value(void)
  {
    return sector;
  }
  
void SD_set_sector_value(unsigned long value)
  {
    sector = value;
  }


void SD_seek_sof()
  {
    sector = 0;
    offset = 0;
  }

RECORD_TYPE* SD_read(unsigned long max_sector)
  {
    static RECORD_TYPE ret;
    int i;
    BYTE status;

    if (offset == 0)
      {
        // we are at the beginning of a new sector
        // load this sector into the buffer
        if (sector>max_sector) return NULL;
        
        status = SectorRead(sector, buffer);
        if (status != TRUE) return NULL;

        // check for last sector 
      }

    memcpy(&ret, (const void*) (buffer+offset), RECORD_SIZE);
    offset += RECORD_SIZE;
    
    // do we need to increment to the next sector?
    if ((offset+RECORD_SIZE) > SECTOR_SIZE)
      {
        sector++;
        offset=0;
      }

    return &ret;
  }
  

int SD_erase()
  {
    int ret = 0;

    SD_CUSTOM_Sector_Erase(0, SECTOR_COUNT_MAX);
    
    // move to beginning
    SD_seek_sof();

    return ret;
  }


// flushData() return 1=good  0=bad
int SD_flush()
  {
    BYTE i;

    LED_YELLOW=LED_ON;
    
    if (sector > SECTOR_COUNT_MAX) return 0;
    
    // save current sector    
    i=3;
    while( (! SD_CUSTOM_SectorWrite2(sector, buffer)) && (--i) ); 

      
    if (i==0) return 0; // this is really bad

    // point to next sector 
    sector++;

    offset = 0; // reset offset 

     // clean the buffer
    memset(buffer,0x00,SECTOR_SIZE);
    LED_YELLOW=LED_OFF;
    return 1;
  }

int SD_write(RECORD_TYPE *ptr)
  {
    // copy record to buffer
    memcpy(buffer+offset, (const void*) ptr, RECORD_SIZE);

    offset += RECORD_SIZE; // increment to next record

    // will the next record fit?
    if ((offset+RECORD_SIZE) > SECTOR_SIZE )
      {
        // let's flush the current buffer
        if (! SD_flush())
          return 0;
      }
    return 1;
  }



