
/** INCLUDES *******************************************************/

#include <p18f4550.h>
#include <timers.h>
#include <adc.h>
#include <pwm.h>
#include "sensors.h"       
#include "led_disp.h" 


/* You will need to configure these for your oscillator */
#pragma config PLLDIV=5
#pragma config CPUDIV=OSC1_PLL2
#pragma config USBDIV=2
#pragma config FOSC=HSPLL_HS

#pragma config FCMEN=OFF
#pragma config IESO=OFF
#pragma config VREGEN=ON
#pragma config CCP2MX=ON

#pragma config WDT=OFF
#pragma config WDTPS=32768

#pragma config PBADEN=OFF
#pragma config PWRT=OFF
#pragma config MCLRE=ON
#pragma config LPT1OSC=OFF
#pragma config BOR=ON		
#pragma config BORV=3		
#pragma config STVREN=ON
#pragma config LVP=OFF
#pragma config ICPRT= OFF
#pragma config XINST=OFF
#pragma config DEBUG=OFF
#pragma config WRTB=ON


/** VARIABLES ******************************************************/
#pragma udata

#define OUT_OF_RANGE_LED PORTBbits.RB0
#define OOR_LED OUT_OF_RANGE_LED

#define TRIS TRISBbits.TRISB0=0

#define OOR_ON  1
#define OOR_OFF 0

extern unsigned int VALUE_VFREQ;
extern unsigned int VALUE_VSOURCE;



volatile int tmr0_overflow = 0;



/** PRIVATE PROTOTYPES *********************************************/

void LowISR(void);
void HighISR(void);

#pragma code

#pragma interruptlow LowISR
void LowISR(void)
  {
  
  }

#pragma interruptlow HighISR
void HighISR(void)
  {	
  
  	//Check which interrupt flag caused the interrupt.
  	if (INTCONbits.TMR0IF)
  	  {
  	  /* The TMR1 register pair (TMR1H:TMR1L) increments
        * from 0000h to FFFFh and rolls over to 0000h. The
        * Timer1 interrupt, if enabled, is generated on overflow,
        * which is latched in interrupt flag bit, TMR1IF
        * (PIR1<0>). This interrupt can be enabled or disabled
        * by setting or clearing the Timer1 Interrupt Enable bit,
        * TMR1IE (PIE1<0>). 
        */
 
   		INTCONbits.TMR0IF = 0;     // clear the interrupt
      tmr0_overflow = 1;   			
  	}  	
 
	//Check which interrupt flag caused the interrupt.
	if (PIR1bits.TMR1IF)
	  {
		  /* The TMR1 register pair (TMR1H:TMR1L) increments
       * from 0000h to FFFFh and rolls over to 0000h. The
       * Timer1 interrupt, if enabled, is generated on overflow,
       * which is latched in interrupt flag bit, TMR1IF
       * (PIR1<0>). This interrupt can be enabled or disabled
       * by setting or clearing the Timer1 Interrupt Enable bit,
       * TMR1IE (PIE1<0>). 
       */

  		PIR1bits.TMR1IF = 0;     // clear the interrupt
		}
  }


#pragma code lowVector=0x18
void LowVector(void){_asm goto LowISR _endasm}

#pragma code highVector=0x8
void HighVector(void){_asm goto HighISR _endasm}

#pragma code  // Return to default code section

void SetDC(unsigned int dutycycle)
{
  union PWMDC DCycle;
  
  // The PWM duty cycle is specified by writing to the
  // CCPR1L register and to the CCP1CON<5:4> bits. Up
  // to 10-bit resolution is available. The CCPR1L contains
  // the eight MSbs and the CCP1CON<5:4> contains the
  // two LSbs. This 10-bit value is represented by
  // CCPR1L:CCP1CON<5:4>.

  // Save the dutycycle value in the union
  DCycle.lpwm = dutycycle << 6;


  // Write the high byte into CCPR1L
  CCPR1L = DCycle.bpwm[1];

  // Write the low byte into CCP1CON5:4
  CCP1CON = (CCP1CON & 0xCF) | ((DCycle.bpwm[0] >> 2) & 0x30);
}
/*
void SetDeadBandDelay(unsigned int d)
  {
    if (d > 0b01111111)  d=0b01111111;
    ECCP1DEL = (ECCP1DEL & 0b10000000) | (0b01111111 & d);
  }
*/
void main(void)
  {   
    unsigned int old_pr2 = 0u;   
    unsigned int new_pr2 = 0u;
    
    unsigned int new_dc  = 0u;
    unsigned int inst_per_period = 0u;
    int clicks = 0;
    int display_value;      
    
  
    ADCON1 |= 0x0F;                 // Default all pins to digital
    
    mike_init_display();
 

    // initialize sensors 
    sensors_init();
    
    // initialize timer
    // http://www.camotruck.net/rollins/electronics/pic_timer.html
    OpenTimer0( 
        TIMER_INT_ON & 
        T0_SOURCE_INT &
        T0_16BIT &
        T0_PS_1_1  // 1 => 0.13 seconds
      ); 
          
    // INTCON=0;            //make sure interrupts are disable
    // INTCONbits.GIE=1;    //enable global interrupts
    // INTCONbits.PEIE=1;   //enable peripheral interrupts
    // INTCONbits.TMR0IE=1; //enable TMR0 overflow interrupt enable bit
    INTCON=0xe0;            //all 4 lines above combined
 
 
    /***** Configure PWM ******/
    
    // 1. Configure the PWM pins, P1A and P1B (and
    //    P1C and P1D, if used), as inputs by setting the
    //    corresponding TRIS bits.
  
    TRISCbits.TRISC2 = 1; // P1A
    TRISDbits.TRISD5 = 1; // P1B
       
    // 2. Set the PWM period by loading the PR2 register.
    //    TOSC = 1/Clock Frequency
    //    PWM Period = [(PR2) + 1]  4  TOSC  (TMR2 Prescale Value)
    
    PR2 = 128;    // 0 to 255
    
    // 4. Configure the ECCP module for the desired
    //    PWM mode and configuration by loading the
    //    CCP1CON register with the appropriate values:
    //    - Select one of the available output
    //      configurations and direction with the
    //      P1M1:P1M0 bits.
    //    - Select the polarities of the PWM output
    //      signals with the CCP1M3:CCP1M0 bits.
    
    // CCP1CON 7:6  (Half-Bridge) = 10
    
    CCP1CON = 0b10001100; // half bridge/active high
    //CCP1CON = 0b10001111; // half bridge/active low
    
    // 5. Set the PWM duty cycle by loading the CCPR1L
    //    register and CCP1CON<5:4> bits.
    
    SetDC(256); // 0 to 1023
    
    // 6. For Half-Bridge Output mode, set the
    //    dead-band delay by loading ECCP1DEL<6:0>
    //    with the appropriate value.
    //   
    //    Bits PDC6:PDC0 of the ECCP1DEL register (Register 16-2) set the delay
    //    period in terms of microcontroller instruction cycles (TCY or 4 TOSC).
    
    ECCP1DEL = (ECCP1DEL & 0b10000000);
    
  
    // 9. Configure and start TMR2:
    //    - Clear the TMR2 interrupt flag bit by clearing
    //      the TMR2IF bit (PIR1<1>).
    //    - Set the TMR2 prescale value by loading the
    //      T2CKPS bits (T2CON<1:0>).
    //    - Enable Timer2 by setting the TMR2ON bit (T2CON<2>).
      
      

    // Timer1 counts the pulses from the external circuit.
    OpenTimer1( 
        TIMER_INT_ON & 
        T1_16BIT_RW &      // max count of 65535    
        T1_SOURCE_EXT &  
        T1_PS_1_1 &   // options are 1,2,4,8
        T1_OSC1EN_OFF &
        T1_SYNC_EXT_OFF 
      );
      
    // Timer2 for PWM
    OpenTimer2( 
        TIMER_INT_OFF & 
        T2_PS_1_16    // 1:1, 1:4, 1:16
      ); 
                
    // 10.Enable PWM outputs after a new PWM cycle
    //    has started:
    //    - Wait until TMRn overflows (TMRnIF bit is set).
      
      // ?
      
    //    - Enable the CCP1/P1A, P1B, P1C and/or P1D
    //      pin outputs by clearing the respective TRIS
    //      bits.
    
    TRISCbits.TRISC2 = 0; // P1A
    TRISDbits.TRISD5 = 0; // P1B
    
    //    - Clear the ECCPASE bit (ECCP1AS<7>).
    //      The modules output mode and polarity are
    //      configured by setting the P1M1:P1M0 and
    //      CCP1M3:CCP1M0 bits of the CCP1CON register.
  
    CCP1ASbits.ECCPASE = 0;


    while(1)
      {
        sensors_update();
        
        // check for voltage of supply
        // adjust frequency
        /* update PWM */
        new_pr2 = (unsigned int) (VALUE_VFREQ/4);
        
       
        
        if ( ((new_pr2>old_pr2)?(new_pr2-old_pr2):(old_pr2-new_pr2)) > 5u )
             {
               display_value = clicks;
               mike_update_display(display_value); 
               old_pr2 = new_pr2;
               
                 PR2 = new_pr2;
        
                 new_dc = 2*(new_pr2+1);
                 SetDC( new_dc );
              }
              
      
        
        
        // PWM Period   = [(PR2) + 1]  4  TOSC  (TMR2 Prescale Value)     (seconds/pwm_cycle)
        // INST_PER_SEC = Clock/4                                            (instruction/second)
        //
        // INST_PER_PWM_CYCLE = [PWM Period (sec/pwm cycle)] * (INST_PER_SEC instruction/sec)
        //                    = [(PR2) + 1]  4  TOSC  (TMR2 Prescale Value) * Clock/4
        //                    = [(PR2) + 1] * TOSC * (TMR2 Prescale Value) * Clock
        //                    = [(PR2) + 1] * (1/Clock) * (TMR2 Prescale Value) * Clock
        //                    = [(PR2) + 1] * (TMR2 Prescale Value)
        //                    = (new_pr2+1)*4;
        //
        // 
        
      //  SetDC( new_dc );


        if (tmr0_overflow)
          {
            tmr0_overflow=0;
            clicks = ReadTimer1();
            WriteTimer1(0);
            
            if ( ((display_value>clicks)?(display_value-clicks):(clicks-display_value)) > 2 )
              {
               display_value = clicks;
               mike_update_display(display_value);   
              }
          }          
        
        mike_refresh_display();
        
        /*
        // if the voltage goes above the upper limit then reduce pulse width
        if (VALUE_VSOURCE > VSOURCE_LIMIT_HIGH)
          {
            SetDeadBandDelay( (new_pr2+1)*2 );  
          }
        else if (VALUE_VSOURCE < VSOURCE_LIMIT_LOW)
          {
            SetDeadBandDelay( 0u );
          }          
          */     
        
      }
  }

