

#include <p18f4550.h>
#include <adc.h>
#include "sensors.h"

#define CH_VSOURCE   ADC_CH0
#define CH_VFREQ     ADC_CH1           
       

#define STATE_VSOURCE_NEW    1        
#define STATE_VSOURCE_BUSY   2
#define STATE_VSOURCE_READ   3

#define STATE_VFREQ_NEW    4        
#define STATE_VFREQ_BUSY   5
#define STATE_VFREQ_READ   6

// #define STATE_VBUCK_NEW      4        
// #define STATE_VBUCK_BUSY     5
// #define STATE_VBUCK_READ     6



#pragma udata

unsigned int VALUE_VSOURCE=0u;
unsigned int VALUE_VFREQ=0u;


static unsigned char state = STATE_VSOURCE_NEW; // private variable
    
#pragma code

void sensors_init(void)
  {
    // Initialize voltage sensors
    OpenADC( 
        ADC_FOSC_2 // clock
        & ADC_RIGHT_JUST //Results in least significant bits
        & ADC_16_TAD // collection time interval
        ,
        CH_VSOURCE & CH_VFREQ // & CH_VBOOST & CH_VMAIN & CH_VLUCK
        & ADC_INT_OFF // not interrupts (use polling)
        & ADC_VREFPLUS_VDD // use +5 for reference voltage
        & ADC_VREFMINUS_VSS // use 0 for reference
        ,
        10 // bits 0 through "n" of the ADCON1 register
      );
      
  }

  
void sensors_update(void)
  {
    switch (state)
      {
        case STATE_VSOURCE_NEW:
            SetChanADC( CH_VSOURCE );
            ConvertADC(); 
            state = STATE_VSOURCE_BUSY;            
          break;
          
        case STATE_VSOURCE_BUSY:
            if (! BusyADC())
              state = STATE_VSOURCE_READ; 
          break;
          
        case STATE_VSOURCE_READ:
            VALUE_VSOURCE = ReadADC();
            state = STATE_VFREQ_NEW; 
          break; 
        
        // --
        
        case STATE_VFREQ_NEW:
            SetChanADC( CH_VFREQ );
            ConvertADC(); 
            state = STATE_VFREQ_BUSY;            
          break;
          
        case STATE_VFREQ_BUSY:
            if (! BusyADC())
              state = STATE_VFREQ_READ; 
          break;
          
        case STATE_VFREQ_READ:
            VALUE_VFREQ = ReadADC();
            state = STATE_VSOURCE_NEW; 
          break;  
          
                
      }
    return;
  }
  

