

#include <p18f4550.h>
#include <adc.h>
#include "sensors.h"
#include "user.h"
#include "GenericTypeDefs.h"

BYTE sensor_update_complete = 0;

#define CH_VSOURCE   ADC_CH0
#define CH_VSHUNT    ADC_CH1           

#define STATE_VSOURCE_NEW    1        
#define STATE_VSOURCE_BUSY   2
#define STATE_VSOURCE_READ   3
#define STATE_VSOURCE_PAUSE  7

#define STATE_VSHUNT_NEW      4        
#define STATE_VSHUNT_BUSY     5
#define STATE_VSHUNT_READ     6
#define STATE_VSHUNT_PAUSE    8


#pragma udata

unsigned int SENSOR_VALUE=0u;


static unsigned char state = STATE_VSOURCE_NEW; // private variable
    
#pragma code

void sensors_init(void)
  {
    // Initialize voltage sensors
    OpenADC( 
        ADC_FOSC_64 // clock
        & ADC_RIGHT_JUST //Results in least significant bits
        & ADC_20_TAD // collection time interval
        ,
        CH_VSOURCE & CH_VSHUNT 
        & ADC_INT_OFF // not interrupts (use polling)
        & ADC_VREFPLUS_VDD // use +5 for reference voltage
        & ADC_VREFMINUS_VSS // use 0 for reference
        ,
        13 // bits 0 through "n" of the ADCON1 register
      );
      
  }

  
void sensors_start()
  {
    sensor_update_complete = 0;
    if (P_MODE_CURRENT)
      state = STATE_VSHUNT_NEW;
    else
      state = STATE_VSOURCE_NEW;
  }
  
void sensors_update(void)
  {
    // Modify this to check on the voltage or only the current
    switch (state)
      {
        case STATE_VSOURCE_NEW:
            SetChanADC( CH_VSOURCE );
            ConvertADC(); 
            state = STATE_VSOURCE_BUSY;            
          break;
          
        case STATE_VSOURCE_BUSY:
            if (! BusyADC())
              state = STATE_VSOURCE_READ; 
          break;
          
        case STATE_VSOURCE_READ:
            SENSOR_VALUE = ReadADC();
            state = STATE_VSOURCE_PAUSE; 
            sensor_update_complete = 1;
          break; 
          
        case STATE_VSOURCE_PAUSE:
            state = STATE_VSOURCE_NEW; 
          break;
          
        
        
        
        case STATE_VSHUNT_NEW:
            SetChanADC( CH_VSHUNT );
            ConvertADC(); 
            state = STATE_VSHUNT_BUSY;            
          break;
          
        case STATE_VSHUNT_BUSY:
            if (! BusyADC())
              state = STATE_VSHUNT_READ; 
          break;
          
        case STATE_VSHUNT_READ:
            SENSOR_VALUE = ReadADC();
            state = STATE_VSHUNT_PAUSE; 
            sensor_update_complete = 1;
          break; 

        case STATE_VSHUNT_PAUSE:
            state = STATE_VSHUNT_NEW; 
          break;
          

      }
    return;
  }
  

